@@ -249,14 +249,14 @@ do.meta.ini <- function(meta, tfr.with.regions, proposal_cov_gammas = NULL,
     lower_U_c <- ifelse(tfr_min_c > meta$U.c.low.base, tfr_min_c, meta$U.c.low.base)
 	prop_cov_gammas <- array(NA, c(nr_countries,3,3))
 	if(use.average.gammas.cov) {
-		cov.to.average <- get.cov.gammas(sim.dir=meta$output.dir, burnin=burnin)$values
-		if (all(is.na(cov.to.average))) {
-			warning('Covariance of gamma is NA for all countries. Average from default covariance will be used.', 
+			cov.to.average <- get.cov.gammas(sim.dir=meta$output.dir, burnin=burnin)$values
+			if (all(is.na(cov.to.average))) {
+				warning('Covariance of gamma is NA for all countries. Average from default covariance will be used.', 
 						immediate.=TRUE)
+			}
 			e <- new.env()
 			data('proposal_cov_gammas_cii', envir=e)
 			cov.to.average <- e$proposal_cov_gammas_cii$values
-		}
 	} else {
 		# get default proposal_cov_gammas_cii and match with country codes of this run
 		e <- new.env()
@@ -558,4 +558,21 @@ mcmc.ini.extra <- function(mcmc, countries, index.replace=NULL) {
 		mcmc[[item]] <- mcmc.update[[item]]
 	}
 	return(mcmc)
-}
\ No newline at end of file
+}
+
+mcmc.meta.ini.subnat <- function(meta, country,
+                                 start.year=1950, present.year=2010, 
+                                 my.tfr.file = NULL, buffer.size=1000,
+                                 verbose=FALSE
+                                ) {
+  # Initialize meta parameters - those that are common to all chains.
+  meta$start.year <- start.year
+  meta$present.year <- present.year
+  meta$buffer.size <- buffer.size
+  tfr.with.regions <- set.wpp.subnat(country=country, start.year=start.year, present.year=present.year,  
+                                     my.tfr.file = my.tfr.file, verbose=verbose)
+  this.meta <- do.meta.ini(meta, tfr.with.regions, verbose=verbose)
+  for (item in names(meta))
+    if(!(item %in% names(this.meta))) this.meta[[item]] <- meta[[item]]
+  return(structure(this.meta, class='bayesTFR.mcmc.meta'))
+}
